#ifndef _diffusion_svc_notify_subscription_topic_specification_h_
#define _diffusion_svc_notify_subscription_topic_specification_h_ 1

/*
 * Copyright © 2017 Push Technology Ltd., All Rights Reserved.
 *
 * Use is subject to license terms.
 *
 * NOTICE: All information contained herein is, and remains the
 * property of Push Technology. The intellectual and technical
 * concepts contained herein are proprietary to Push Technology and
 * may be covered by U.S. and Foreign Patents, patents in process, and
 * are protected by trade secret or copyright law.
 */

/**
 * @file svc-notify-subscription-topic-specification.h
 *
 * Functions for notifying a client that it has been subscribed to a
 * topic.
 */

#include "internal/service_int.h"
#include "topic-specification.h"

/**
 * @brief Structure describing an incoming notification that the
 * client has been subscribed to a topic.
 */
typedef struct svc_notify_subscription_topic_specification_request_s {
        /// Topic identification.
        struct {
                /// Internal identifier for the topic.
                uint32_t        topic_id;
                /// Name of the topic which the client was subscribed
                /// to.
                char *          topic_path;
        } topic_info;
        /// Description of the structure of the topic.
        TOPIC_SPECIFICATION_T *specification;
} SVC_NOTIFY_SUBSCRIPTION_TOPIC_SPECIFICATION_REQUEST_T;

/**
 * @brief Response message to subscription notifications.
 */
typedef struct svc_notify_subscription_topic_specification_response_s {
        /// @cond HIDDEN
        void *_reserved;
        /// @endcond
} SVC_NOTIFY_SUBSCRIPTION_TOPIC_SPECIFICATION_RESPONSE_T;

DECLARE_SERVICE_PROTOTYPES(notify_subscription_topic_specification, SVC_NOTIFY_SUBSCRIPTION_TOPIC_SPECIFICATION_REQUEST_T, SVC_NOTIFY_SUBSCRIPTION_TOPIC_SPECIFICATION_RESPONSE_T)

#endif
