#ifndef _diffusion_svc_update_value_h_
#define _diffusion_svc_update_value_h_ 1

/*
 * Copyright © 2016 Push Technology Ltd., All Rights Reserved.
 *
 * Use is subject to license terms.
 *
 * NOTICE: All information contained herein is, and remains the
 * property of Push Technology. The intellectual and technical
 * concepts contained herein are proprietary to Push Technology and
 * may be covered by U.S. and Foreign Patents, patents in process, and
 * are protected by trade secret or copyright law.
 */

/**
 * @file svc-update-value.h
 *
 * Functions for providing updates to topics. Registration is controlled with
 * register_update_source() and deregister_update_source(), but the
 * update_value() function provided here is capable of transparently receiving
 * and sending more efficient binary deltas of change between updates.
 *
 */

#include "internal/service_int.h"
#include "types/update_types.h"
#include "update.h"
#include "datatypes.h"
#include "service/svc-update.h"

typedef struct update_value_params_s {
        UPDATE_SOURCE_HANDLERS
        CONVERSATION_ID_T *updater_id;
        char *topic_path;
        BUF_T *data;
        void *context;
} UPDATE_VALUE_PARAMS_T;

typedef struct svc_update_source_set_request_s {
        CONVERSATION_ID_T *updater_id;
        char *topic_path;
        BUF_T *data;
} SVC_UPDATE_SOURCE_SET_REQUEST_T;

typedef struct svc_update_source_delta_request_s {
        CONVERSATION_ID_T *updater_id;
        char *topic_path;
        BUF_T *data;
        uint32_t delta_id;
} SVC_UPDATE_SOURCE_DELTA_REQUEST_T;

/**
 * @ingroup PublicAPI_TopicUpdateControl
 *
 * @brief Update topic content
 *
 * Updates the content of a topic of a certain type (binary, JSON) but unlike
 * update(), this may transparently send delta updates which require less
 * network bandwidth. If possible, prefer this function to update().
 *
 * @param session	The session handle. If NULL, this function returns immediately.
 * @param params        Parameter structure describing which topic to update, the value
 *                      that the topic on the server will contain and callbacks to
 *                      handle success or failure notifications.
 */
void update_value(SESSION_T *session, const UPDATE_VALUE_PARAMS_T params);

/**
 * @ingroup PublicAPI_TopicUpdateControl
 *
 * @brief Update topic content
 *
 * Updates the content of a topic of a provided datatype. This is the preferred function
 * for updating datatype topics (Binary, JSON, String, Int64, Double and Recordv2)
 *
 * @param session	The session handle. If NULL, this function returns immediately.
 * @param datatype      `DIFFUSION_DATATYPE` of the topic to be updated.
 * @param params        Parameter structure describing which topic to update, the value
 *                      that the topic on the server will contain and callbacks to
 *                      handle success or failure notifications.
 */
void update_value_with_datatype(SESSION_T *session, DIFFUSION_DATATYPE datatype, UPDATE_VALUE_PARAMS_T params);

DECLARE_SERVICE_PROTOTYPES(update_source_set, SVC_UPDATE_SOURCE_SET_REQUEST_T, SVC_UPDATE_RESPONSE_T)
DECLARE_SERVICE_PROTOTYPES(update_source_delta, SVC_UPDATE_SOURCE_DELTA_REQUEST_T, SVC_UPDATE_RESPONSE_T)

#endif
